/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: Help
 * Description: -
 */

#define Uses_Ui
#define Uses_DataFile
#define Uses_Career
#include "mheader.h"

#include "help.h"



typedef struct help_file_data_struct	HELP_FILE_DATA;


/*
 * help file data structure
 */
struct help_file_data_struct {

	/* title of the help file */
	char *		title;

	/* file name of the help file */
	const char *	file_name;

};



static void		additional_information_screen(void);

static void		show_career_list(void);



/*
 * help file data
 */
static const HELP_FILE_DATA	HelpFile[MAX_HELP_FILES] = {

	{"Stats",			FILE_HELP_STATS},
	{"Perks",			FILE_HELP_PERKS},
	{"Status",			FILE_HELP_STATUS},
	{"Parties",			FILE_HELP_PARTIES},
	{"Advancement",			FILE_HELP_ADVANCEMENT},
	{"Careers",			FILE_HELP_CAREERS},
	{"Timing",			FILE_HELP_TIMING},
	{"Movement",			FILE_HELP_MOVEMENT},
	{"Combat",			FILE_HELP_COMBAT},
	{"Evocation",			FILE_HELP_EVOCATION},
	{"Psychic Powers",		FILE_HELP_PSYCHIC_POWERS},
	{"Tactical Commands",		FILE_HELP_TACTICS},
	{"Objects",			FILE_HELP_OBJECTS},
	{"Terrain",			FILE_HELP_TERRAIN},
	{"Macros",			FILE_HELP_MACROS},
	{"Hints",			FILE_HELP_HINTS}

};



/*
 * the help screen
 */
void help_screen(void)
{

	command_bar_set(2, CM_ADDITIONAL_INFORMATION, CM_EXIT);

	render_help_screen();

	update_screen();

	if (command_bar_get_command() == CM_EXIT) {

		return;
	}

	additional_information_screen();
}



/*
 * the additional information screen
 */
static void additional_information_screen(void)
{
	LIST *items;
	MENU *menu;
	FILE_COUNTER i;

	items = list_new();

	for (i = 0; i < MAX_HELP_FILES; i++) {

		list_add(items, HelpFile[i].title);
	}

	menu = menu_create(items);

	while (true) {
		command_bar_set(4, CM_UP, CM_DOWN, CM_OK, CM_EXIT);

		render_additional_information_screen();

		additional_information_menu(menu);

		if (menu->entered_command == CM_EXIT) {

			break;
		}
		
		/* hack */
		if (HelpFile[menu->highlighted].file_name == NULL) {
			
			show_career_list();

			continue;
		}

		show_text_file(
			HelpFile[menu->highlighted].title,
			HelpFile[menu->highlighted].file_name,
			false
		);
	}

	menu_destroy(menu);
}



/*
 * shows a list of all careers
 */
static void show_career_list(void)
{
	MENU *menu;
	LIST *items;
	N_CAREERS n_careers;
	CAREER_COUNTER i;

	n_careers = get_n_careers();
	items = list_new();

	for (i = 0; i < n_careers; i++) {
		CAREER *career;

		career = get_career_pointer(i);

		list_add(items, (void *)career->name);
	}

	menu = menu_create(items);

	do {

		command_bar_set(5, CM_UP, CM_DOWN, CM_INFO, CM_OK,
			CM_EXIT
		);

		render_show_career_list_screen();
		update_screen();

		show_career_list_menu(menu);

		if (menu->entered_command == CM_INFO) {
			CAREER_INDEX career_index;
			const char *career_name;
			
			career_name = (const char *)list_data_at(menu->items,
				menu->highlighted
			);

			career_index = name_to_career_index(career_name);

			career_screen(get_career_pointer(career_index));
		}

	} while (menu->entered_command != CM_EXIT);
	
	menu_destroy(menu);
}


