/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: HallOfFame
 * Description: -
 */


#define Uses_Ui
#define Uses_Util
#define Uses_Character
#define Uses_LoadSave
#define Uses_Game
#include "mheader.h"

#include "hof.h"


#define MAX_HALL_OF_FAME_ENTRIES	7

#define HOF_WINNER_BONUS		100000

#define IRON_MAN_SCORE_MULTIPLIER	2




static LIST *			HallOfFame = NULL;


static HALL_OF_FAME_ENTRY *	hof_entry_create(const CHARACTER *, bool);
static HALL_OF_FAME_ENTRY *	hof_entry_clone(HALL_OF_FAME_ENTRY *);

static SCORE			score(const CHARACTER *, bool);




/*
 * hall of fame module init
 */
void hall_of_fame_init(void)
{

	HallOfFame = list_new();

	hall_of_fame_read();
}



/*
 * hall of fame module clean up
 */
void hall_of_fame_clean_up(void)
{

	if (HallOfFame == NULL) return;

	hall_of_fame_write();

	list_destroy_with(HallOfFame, free);
}



/*
 * returns the hall of fame
 */
LIST * hall_of_fame(void)
{

	return HallOfFame;
}



/*
 * returns a hall of fame entry
 */
const HALL_OF_FAME_ENTRY * hall_of_fame_entry(int n)
{

	return (const HALL_OF_FAME_ENTRY *)list_data_at(HallOfFame, n);
}



/*
 * the hall of fame screen
 */
void hall_of_fame_screen(void)
{

	command_bar_set(1, CM_EXIT);

	render_hall_of_fame_screen();
	update_screen();

	command_bar_get_command();
}



/*
 * attempts to add a character to the hall of fame
 * returns true if the character has been added or
 * false if he hasn't been added
 */
bool hall_of_fame_add(const CHARACTER *character, bool winner)
{
	LIST_NODE *node;
	HALL_OF_FAME_ENTRY *new_entry;
	LIST *new_hof;
	int i;

	node = HallOfFame->head;

	/* special case - HOF is empty */
	if (node == NULL) {

		new_entry = hof_entry_create(character, winner);

		list_add(HallOfFame, new_entry);

		return true;
	}

	new_hof = list_new();
	new_entry = NULL;
	i = 0;
	while (node != NULL) {
		HALL_OF_FAME_ENTRY *entry;
		HALL_OF_FAME_ENTRY *entry_clone;

		entry = (HALL_OF_FAME_ENTRY *)node->data;

		if (new_entry == NULL &&
			entry->score <= score(character, winner)) {

			new_entry = hof_entry_create(character, winner);
			list_add(new_hof, new_entry);
			++i;

			if (i == MAX_HALL_OF_FAME_ENTRIES) break;
		}

		entry_clone = hof_entry_clone(entry);

		list_add(new_hof, entry_clone);
		++i;

		if (i == MAX_HALL_OF_FAME_ENTRIES) break;

		node = node->next;
	}

	/* special case - last position */
	if (new_entry == NULL && i < MAX_HALL_OF_FAME_ENTRIES) {

		new_entry = hof_entry_create(character, winner);
		list_add(new_hof, new_entry);
	}

	list_destroy_with(HallOfFame, free);

	HallOfFame = new_hof;

	if (new_entry != NULL) return true;

	return false;
}



/*
 * creates a hall of fame entry
 */
static HALL_OF_FAME_ENTRY * hof_entry_create(const CHARACTER *character, bool winner)
{
	HALL_OF_FAME_ENTRY *hof_entry;

	hof_entry = checked_malloc(sizeof *hof_entry);

	hof_entry->score = score(character, winner);

	strcpy(hof_entry->name, character->name);

	if (game_mode() == GAME_MODE_IRON_MAN) {

		strcat(hof_entry->name, " (Iron man)");
	}

	hof_entry->winner = winner;

	return hof_entry;
}



/*
 * produces an identical copy of a hall of fame entry
 */
static HALL_OF_FAME_ENTRY * hof_entry_clone(HALL_OF_FAME_ENTRY *entry)
{
	HALL_OF_FAME_ENTRY *clone;

	clone = checked_malloc(sizeof *clone);

	*clone = *entry;

	return clone;
}



/*
 * calculates a character's score
 */
static SCORE score(const CHARACTER *character, bool winner)
{
	SCORE result;

	result = character_power_rating(character);

	if (winner) {

		result += HOF_WINNER_BONUS;
	}

	if (game_mode() == GAME_MODE_IRON_MAN) {

		result *= IRON_MAN_SCORE_MULTIPLIER;
	} 

	return result;
}
