/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Dungeon builder (Cell Point)
 *
 *
 * NAMESPACE: CPN_* / cpn_*
 *
 */


/*
 * dungeon tiles
 */
#define CPN_FLOOR			0
#define CPN_DESTRUCTABLE_OBSTACLE	1
#define CPN_DESTRUCTABLE_OBSTACLE_2	2
#define CPN_WALL			3


/*
 * room types
 */
enum cpn_room_type_enum {

	CPN_ROOM_NOT_BUILT,

	CPN_ROOM_CORRIDOR_CONNECTION,

	CPN_ROOM_NORMAL,

	CPN_ROOM_H_DOUBLE,
	CPN_ROOM_V_DOUBLE,
	CPN_ROOM_QUAD

};


typedef int				CPN_TILE;

typedef enum cpn_room_type_enum		CPN_ROOM_TYPE;

typedef struct cpn_rectangle_struct	CPN_RECTANGLE;

typedef struct cpn_room_struct		CPN_ROOM;

typedef struct cpn_dungeon_struct	CPN_DUNGEON;


/*
 * rectanlge
 */
struct cpn_rectangle_struct {

	int		top;
	int		bottom;
	int		left;
	int		right;

};


/*
 * room
 */
struct cpn_room_struct {

	CPN_ROOM_TYPE	type; /* room type */

	/* room source point coordinates */
	int		source_y;
	int		source_x;

	CPN_RECTANGLE	padding;

	CPN_RECTANGLE	area;

	CPN_RECTANGLE	connection;

	int		n_connections;
};


/*
 * dungeon
 */
struct cpn_dungeon_struct {

	int width;
	int height;
	CPN_TILE **	map;

	int n_rooms;
	CPN_ROOM *	room;

};



void		cpn_free_dungeon(CPN_DUNGEON *);

int		cpn_set_cell_size(int, int);

int		cpn_set_minimal_room_size(int, int);

void		cpn_set_rng(int(*)(int, int));

const char *	cpn_get_error(void);

void		cpn_set_corridor_frequency(int);

CPN_DUNGEON *	cpn_build_dungeon(int, int, CPN_ROOM *);

int		cpn_output_dungeon(const char *);



