/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: Manufacture
 * Description: -
 */

#define Uses_Ui
#define Uses_Trade
#define Uses_Object
#define Uses_Inventory
#define Uses_Perks
#define Uses_Util
#define Uses_ProgramManager
#include "mheader.h"

#include "manufact.h"



static MENU *	manufacture_menu_create(const CHARACTER *, 
			LIST *, PRICE_MODE
		);

static void	manufacture_menu_build_object(CHARACTER *, OBJECT *,
			MANUFACTURE_MODE
		);



/*
 * the manufacture screen
 */
void manufacture_screen(CHARACTER *character,
	MANUFACTURE_MODE manufacture_mode
)
{
	MENU *menu;
	LIST *available_objects;
	PRICE_MODE price_mode;
	COMMAND action_command;

	if (manufacture_mode == MM_TRADER) {

		available_objects = object_list_trader();

		price_mode = PRICE_MODE_TRADER;
		
		action_command = CM_BUY;

	} else if (manufacture_mode == MM_ROGUE_TRADER) {

		available_objects = object_list_rogue_trader();

		price_mode = PRICE_MODE_ROGUE_TRADER;
		
		action_command = CM_BUY;

	} else {

		price_mode = PRICE_MODE_MANUFACTURE;

		if (manufacture_mode == MM_DRUGS) {

			available_objects = object_list_drugs(NULL);

		} else {

			available_objects = NULL;

			die("*** CORE ERROR *** invalid manufacture mode");
		}
		
		action_command = CM_MANUFACTURE;
	}

	menu = manufacture_menu_create(character, available_objects,
		price_mode
	);

	do  {
		OBJECT *object;

		command_bar_set(5, CM_UP, CM_DOWN, action_command, 
			CM_INFO, CM_EXIT
		);

		render_manufacture_screen(character, manufacture_mode);

		manufacture_screen_menu(menu);

		object = (OBJECT *)list_data_at(available_objects,
			menu->highlighted
		);

		if (menu->entered_command == action_command) {

			manufacture_menu_build_object(character,
				object, manufacture_mode
			);

		} else if (menu->entered_command == CM_INFO) {

			object_screen(object);
		}

	} while (menu->entered_command != CM_EXIT);

	menu_destroy_with(menu, free);

	list_destroy_with(available_objects, object_destroy);
}



/*
 * creates the manufacture menu
 */
static MENU * manufacture_menu_create(const CHARACTER *character,
	LIST *available_objects, PRICE_MODE price_mode)
{
	LIST *menu_items;
	MENU *menu;
	LIST_NODE *node;

	menu_items = list_new();

	for (node = available_objects->head;
		node != NULL;
		node = node->next) {
		OBJECT *object;
		char *entry;

		object = (OBJECT *)node->data;

		entry = checked_malloc(STRING_BUFFER_SIZE);

		sprintf(entry, "%s (%dc) ",
			object_static_data(object)->name,
			object_price(character, object, price_mode)
		);

		list_add(menu_items, entry);
	}

	menu = menu_create(menu_items);

	return menu;
}



/*
 * manufacture menu - build object
 */
static void manufacture_menu_build_object(CHARACTER *character, OBJECT *object,
	MANUFACTURE_MODE manufacture_mode)
{
	MONEY price;
	PRICE_MODE price_mode;

	if (manufacture_mode == MM_TRADER) {

		price_mode = PRICE_MODE_TRADER;
		
	} else if (manufacture_mode == MM_ROGUE_TRADER) {

		price_mode = PRICE_MODE_ROGUE_TRADER;

	} else {
		
		price_mode = PRICE_MODE_MANUFACTURE;
	}

	price = object_price(character, object, price_mode);

	if (price > inventory_money(character)) {

		message(SID_DIALOG, "Not enough credits");
		
		return;
	}

	if (inventory_is_full(character)) {

		message(SID_DIALOG, "Inventory is full");
		
		return;
	}

	if (object_static_data(object)->weight +
		inventory_weight(character) > ENCUMBRANCE_MAX(character)) {

		message(SID_DIALOG, "You cannot carry any more");
			
		return;
	}

	if (manufacture_mode == MM_TRADER || 
		manufacture_mode == MM_ROGUE_TRADER) {

		message(SID_DIALOG, "Bought");

	} else {

		message(SID_DIALOG, "Built");
	}

	inventory_money_reduce(character, price);

	inventory_add(character, object_clone(object));
}

