/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: Npc
 * Description: -
 */

#define Uses_Ui
#define Uses_Character
#define Uses_Faction
#define Uses_Career
#define Uses_Stats
#define Uses_Util
#define Uses_ProgramManager
#define Uses_Psychic
#define Uses_Perks
#define Uses_Inventory
#define Uses_Object
#define Uses_Equipment
#include "mheader.h"

#include "npc.h"


#define NPC_BOX_BUFFER_SIZE		50


static void		npc_randomize_stats(CHARACTER *);



/*
 * the NPC box
 */
static CHARACTER *	NpcBox = NULL;
static N_NPCS		NpcBoxCurrentSize = 0;
static N_NPCS		NpcBoxMaxSize = 0;





/*
 * module clean up
 */
void npc_clean_up(void)
{
	int i;

	for (i = 0; i < NpcBoxCurrentSize; i++) {
		CHARACTER *npc;

		npc = &NpcBox[i];

		list_destroy_with(npc->inventory, object_destroy);

		list_destroy(npc->noticed_enemies);
	}

	free(NpcBox);
}



/*
 * adds a new NPC to the NPC box
 */
CHARACTER * npc_box_new_npc(void)
{
	CHARACTER *npc;

	++NpcBoxCurrentSize;

	if (NpcBoxCurrentSize > NpcBoxMaxSize) {
		NpcBoxMaxSize += NPC_BOX_BUFFER_SIZE;

		NpcBox = checked_realloc(NpcBox,
			NpcBoxMaxSize *
			sizeof *NpcBox
		);
	}

	npc = &NpcBox[NpcBoxCurrentSize - 1];

	character_init(npc);

	npc->inventory = list_new();
	npc->noticed_enemies = list_new();

	return npc;
}



/*
 * optimizes the NPC box
 */
void npc_box_optimize(void)
{

	if (NpcBoxCurrentSize < NpcBoxMaxSize) {
		NpcBoxMaxSize = NpcBoxCurrentSize;

		NpcBox = checked_realloc(NpcBox,
			NpcBoxMaxSize *
			sizeof *NpcBox
		);
	}
}



/*
 * creates an NPC (name based)
 */
CHARACTER * npc_create(const char *npc_name)
{
	NPC_INDEX npc_index;
	CHARACTER *npc;

	npc_index = name_to_npc_index(npc_name);

	npc = npc_create_i(npc_index);

	return npc;
}



/*
 * creates an NPC (index based)
 */
CHARACTER * npc_create_i(NPC_INDEX npc_index)
{
	CHARACTER *npc;

	npc = character_clone(&NpcBox[npc_index]);

	if (!character_has_flag(npc, CF_UNIQUE)) {

		npc_randomize_stats(npc);
	}

	return npc;
}



/*
 * returns true if the passed NPC is unique
 */
bool npc_is_unique(NPC_INDEX npc_index)
{

	if (npc_index == NPC_RANDOM_HUMAN) {
		
		return false;
	}

	return NpcBox[npc_index].flag[CF_UNIQUE];
}



/*
 * name -> NPC index
 */
NPC_INDEX name_to_npc_index(const char *name)
{
	int i;

	for (i = 0; i < NpcBoxCurrentSize; i++) {

		if (strings_equal(name, NpcBox[i].name)) {

			return i;
		}
	}

	die("*** CORE ERROR *** invalid NPC (%s)", name);

	return 0;
}



/*
 * adds some random variation to a NPC's stats
 */
static void npc_randomize_stats(CHARACTER *character)
{
	STAT_COUNTER i;

	for (i = 0; i < MAX_STATS; i++) {
		CHARACTER_STAT *stat;
		int p;

		stat = &character->stat[i];

		if (stat->total == 0) {
			
			continue;
		}
		
		p = random_int(80, 120);
		
		stat->total = percent(p, stat->total);

		stat->current = stat->total;
	}
}


