/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: Trade
 * Description: -
 */

#define Uses_Inventory
#define Uses_Manufacture
#define Uses_ProgramManager
#define Uses_Util
#define Uses_Object
#define Uses_Perks
#include "mheader.h"

#include "trade.h"



/*
 * the buy screen
 */
void buy_screen(CHARACTER *buyer, CHARACTER *seller)
{

	if (seller->perk[PK_TRADING]) {

		manufacture_screen(buyer, MM_TRADER);

	} else if (seller->perk[PK_ROGUE_TRADING]) {

		manufacture_screen(buyer, MM_ROGUE_TRADER);

	} else {

		die("*** CORE ERROR ** invalid seller");
	}
}



/*
 * the sell screen
 */
void sell_screen(CHARACTER *character)
{

	inventory_screen(character, true);
}



/*
 * sells an object
 */
void sell_object(CHARACTER *character, OBJECT *object)
{

	inventory_money_increase(character,
		object_price(character, object, PRICE_MODE_SELL)
	);
}



/*
 * returns the price of an object (dependent on price mode)
 */
MONEY object_price(const CHARACTER *character, const OBJECT *object,
	PRICE_MODE price_mode
)
{
	MONEY value, price;
	int p;

	if (price_mode == PRICE_MODE_SELL ||
		price_mode == PRICE_MODE_MANUFACTURE) {

		p = SELL_PRICE;

	} else if (price_mode == PRICE_MODE_TRADER) {

		p = TRADER_PRICE;

	} else if (price_mode == PRICE_MODE_ROGUE_TRADER) {

		p = ROGUE_TRADER_PRICE;

	} else {

		p = 0;

		die("*** CORE ERROR *** invalid price mode");
	}

	
	value = object_static_data(object)->value;
	
	if (value == VALUE_NIL) {
		
		return value;
	}
	
	price = percent(p, value);

	if (character->perk[PK_PERSUADE] &&
		(price_mode == PRICE_MODE_TRADER ||
			price_mode == PRICE_MODE_ROGUE_TRADER)) {

		price = percent(PERSUADE_PRICE, price);
	}

	return price;
}


