/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 *
 * Global types
 *
 */


#if INT_MAX < 2147483647
#	error int is too narrow (must be at least 32bit wide)
#endif


typedef struct area_point_struct		AREA_POINT;
typedef struct area_section_struct		AREA_SECTION;

typedef struct world_point_struct		WORLD_POINT;

typedef struct screen_area_struct		SCREEN_AREA;

typedef struct rgb_data_struct			RGB_DATA;

typedef struct dynamic_color_struct		DYNAMIC_COLOR;

typedef struct sector_struct			SECTOR;

typedef struct path_node_struct			PATH_NODE;

typedef struct dice_roll_struct			DICE_ROLL;

typedef struct event_struct			EVENT;

typedef struct firing_mode_data_struct		FIRING_MODE_DATA;

typedef struct firing_data_struct		FIRING_DATA;

typedef struct attack_data_struct		ATTACK_DATA;

typedef struct character_stat_struct		CHARACTER_STAT;
typedef struct armour_rating_struct		ARMOUR_RATING;
typedef struct ai_data_struct			AI_DATA;
typedef struct career_struct			CAREER;

typedef struct terrain_struct			TERRAIN;
typedef struct terrain_data_struct		TERRAIN_DATA;

typedef struct object_struct			OBJECT;
typedef struct object_data_struct		OBJECT_DATA;

typedef struct character_struct			CHARACTER;

typedef struct area_struct			AREA;

typedef struct menu_struct			MENU;

typedef struct get_text_dialog_struct		GET_TEXT_DIALOG;

typedef struct confirmation_dialog_struct	CONFIRMATION_DIALOG;

typedef struct dialog_state_struct		DIALOG_STATE;




/*********************************************************************/
/*                                                                   */
/* IO types                                                          */
/*                                                                   */
/*********************************************************************/
typedef int				FILE_INDEX;
#define FILE_INDEX_NIL			-1

typedef int				FILE_COUNTER;



/*********************************************************************/
/*                                                                   */
/* UI types                                                          */
/*                                                                   */
/*********************************************************************/
typedef int				SCREEN_COORD;
#define MENU_AUTO			-1

typedef int				KEY_CODE;

typedef int				MACRO_INDEX;

typedef int				COLOR_VALUE;
typedef int				COLOR_FREQUENCY;
#define COLOR_FREQUENCY_MIN		1
#define COLOR_FREQUENCY_MAX		100

typedef int				SYMBOL;

/*
 * screen area rectangle
 */
struct screen_area_struct {

	SCREEN_COORD	top;
	SCREEN_COORD	bottom;
	SCREEN_COORD	left;
	SCREEN_COORD	right;

};

/*
 * RGB data
 */
struct rgb_data_struct {

	COLOR_VALUE	red;
	COLOR_VALUE	green;
	COLOR_VALUE	blue;

};

/*
 * dynamic color
 */
struct dynamic_color_struct {

	COLOR		primary;
	COLOR		secondary;

	COLOR_FREQUENCY	frequency;

};

/*
 * menu
 */
struct menu_struct {

	LIST *		items;

	int		first_visible;
	int 		last_visible;
	int 		highlighted;
	int		last;

	COMMAND		entered_command;

};

/*
 * get text dialog
 */
struct get_text_dialog_struct {

	char		prompt[STRING_BUFFER_SIZE];

	COLOR		prompt_color;

	char		entered_text[STRING_BUFFER_SIZE];

	int		max_length;

};

/*
 * confirmation dialog
 */
struct confirmation_dialog_struct {

	char		prompt[STRING_BUFFER_SIZE];

	bool		confirmed;

};

/*
 * dialog state
 */
struct dialog_state_struct {

	char text[DIALOG_TEXT_SIZE];

	CHARACTER *active_character;

	CHARACTER *passive_character;

	LIST *options;

	MENU *menu;

};







/*********************************************************************/
/*                                                                   */
/* World types                                                       */
/*                                                                   */
/*********************************************************************/
typedef int				WORLD_COORD;


/*
 * world point coordinates
 */
struct world_point_struct {

	WORLD_COORD	z;
	WORLD_COORD	y;
	WORLD_COORD	x;

};









/*********************************************************************/
/*                                                                   */
/* Area types                                                        */
/*                                                                   */
/*********************************************************************/
typedef int				AREA_COORD;
#define AREA_COORD_NIL			-1

typedef int				AREA_DISTANCE;
#define AREA_DISTANCE_MAX		INT_MAX
#define AREA_DISTANCE_NIL		-1

/*
 * area point coordinates
 */
struct area_point_struct {

	AREA_COORD	y;
	AREA_COORD	x;

};

/*
 * area section rectangle
 */
struct area_section_struct {

	AREA_COORD	top;
	AREA_COORD	bottom;
	AREA_COORD	left;
	AREA_COORD	right;

};

/*
 * area sector
 */
struct sector_struct {

	TERRAIN *	terrain;

	OBJECT *	object;

	CHARACTER *	character;

};

/*
 * area
 */
struct area_struct {

	char		name[AREA_NAME_SIZE];

	WORLD_POINT	coord;

	SECTOR *	sector;

	LIST *		characters;
	LIST *		objects;
	LIST *		terrain;

	LIST *		events;

};







/*********************************************************************/
/*                                                                   */
/* Pathfinder types                                                  */
/*                                                                   */
/*********************************************************************/

/*
 * path node
 */
struct path_node_struct {

	AREA_POINT	p;

	PATH_NODE *	parent;

};









/*********************************************************************/
/*                                                                   */
/* Dice types                                                        */
/*                                                                   */
/*********************************************************************/

/*
 * dice roll
 */
struct dice_roll_struct {

	int		n_dice;
	int		n_sides;

	int		modifier;

};






/*********************************************************************/
/*                                                                   */
/* Stat types                                                        */
/*                                                                   */
/*********************************************************************/
typedef int				STAT_VALUE;

typedef int				STAT_MODIFIER;
#define STAT_MODIFIER_MAX		INT_MAX

typedef int				STAT_COUNTER;


/*
 * character stat
 */
struct character_stat_struct {

	STAT_VALUE	current;
	STAT_VALUE	total;
	STAT_MODIFIER	advance;

};








/*********************************************************************/
/*                                                                   */
/* Time types                                                        */
/*                                                                   */
/*********************************************************************/
typedef int				PHASE_TIME;
#define TIME_UNDETERMINED		-1
#define TIME_CONTINUOUS			-2

typedef int				PHASE;





/*********************************************************************/
/*                                                                   */
/* Event types                                                       */
/*                                                                   */
/*********************************************************************/

typedef int			N_EVENTS;

typedef int			EVENT_POWER;

/*
 * event
 */
struct event_struct {

	EVENT_TYPE		type;

	PHASE_TIME		delay;

	CHARACTER *		character;

	OBJECT *		object;

	DRUG			drug;
	PSY_POWER		psy_power;
	EFFECT			effect;

	EVENT_POWER		power;

	STAT_MODIFIER		stat_modifier[MAX_STATS];

};







/*********************************************************************/
/*                                                                   */
/* Combat system types                                               */
/*                                                                   */
/*********************************************************************/
typedef int				N_SHOTS;
typedef int				N_STRIKES;
typedef int				N_TARGETS;

typedef int				HIT_ROLL;
typedef int				HIT_VALUE;
typedef int				HIT_MODIFIER;

typedef int				REACH_MODIFIER;

typedef int				PARRY_VALUE;

typedef int				A_MODE_SPREAD_FACTOR;

typedef int				DAMAGE;

/*
 * firing data
 */
struct firing_data_struct {

	FIRING_MODE			firing_mode;

	N_SHOTS				n_shots;

	A_MODE_SPREAD_FACTOR		spread;

};

/*
 * attack data structure
 */
struct attack_data_struct {

	ATTACK_TYPE		type;

	CHARACTER *		attacker;

	OBJECT *		weapon;

	FIRING_DATA *		firing_data;

	AREA_POINT		target_point;

	AREA_DISTANCE		distance;

	bool			sneak;

	bool			rapid;

	HIT_VALUE		hit_value;

	HIT_ROLL		hit_roll;

	bool			critical_hit;

};







/*********************************************************************/
/*                                                                   */
/* Terrain types                                                     */
/*                                                                   */
/*********************************************************************/
typedef int				TERRAIN_INDEX;

/*
 * terrain
 */
struct terrain_struct {

	TERRAIN_INDEX			index;

	COLOR				color;

	GORE_LEVEL			gore_level;

	LIST_NODE *			list_node;
	LIST_NODE_INDEX			list_index;

};

/*
 * static terrain data
 */
struct terrain_data_struct {

	char 			name[TERRAIN_NAME_SIZE];

	bool			attribute[MAX_TERRAIN_ATTRIBUTES];

	SYMBOL			symbol;

	DYNAMIC_COLOR		color;

};







/*********************************************************************/
/*                                                                   */
/* Object types                                                      */
/*                                                                   */
/*********************************************************************/
typedef int				OBJECT_INDEX;

typedef int				MONEY;
#define VALUE_NIL			-1

typedef int				WEIGHT;
#define WEIGHT_NIL			-1
#define ENCUMBRANCE_MAX(c)		((c)->stat[S_ST].current * 10)

typedef int				WEAPON_REACH;

typedef int				PARRY_PENALTY;

typedef int				CONDITION;
#define CONDITION_MAX			INT_MAX
#define CONDITION_INDESTRUCTABLE	CONDITION_MAX

typedef int				RELOAD_VALUE;

typedef int				ARMOUR_VALUE;

typedef int				CHARGE;
#define	CHARGE_NIL			-1

typedef int				N_OBJECTS;

typedef int				N_OBJECT_ATTRIBUTES;
typedef int				OBJECT_ATTRIBUTE_COUNTER;


/*
 * object firing mode data
 */
struct firing_mode_data_struct {

	bool			has[MAX_FIRING_MODES];

	N_SHOTS			min_sa_shots;
	N_SHOTS			max_sa_shots;

	N_SHOTS			a_shots;

	HIT_MODIFIER		accuracy;

};

/*
 * object
 */
struct object_struct {

	OBJECT_INDEX		index;

	COLOR			color;

	GORE_LEVEL		gore_level;

	CONDITION		condition;

	CHARGE			charge;

	bool			functional;

	LIST_NODE *		list_node;
	LIST_NODE_INDEX		list_index;

};

/*
 * static object data
 */
struct object_data_struct {

	/* common data ****************************/
	char			name[OBJECT_NAME_SIZE];

	bool			attribute[MAX_OBJECT_ATTRIBUTES];

	SYMBOL			symbol;
	DYNAMIC_COLOR		color;

	CONDITION		condition_max;

	CHARGE			charge_max;

	OBJECT_TYPE		type;
	OBJECT_SUBTYPE		subtype;

	WEIGHT			weight;

	MONEY			value;
	bool			buyable;
	/******************************************/

	/* weapons only ***************************/
	DICE_ROLL		damage;
	/******************************************/

	/* ranged combat weapons only *************/
	FIRING_MODE_DATA	firing_mode;
	RELOAD_VALUE		reload;
	/******************************************/

	/* close combat weapons only **************/
	WEAPON_REACH		reach;
	PARRY_PENALTY		parry_penalty;
	/******************************************/

	/* armours only ***************************/
	ARMOUR_VALUE		armour_value;
	/******************************************/

	/* source file ****************************/
	FILE_INDEX		source_file;
	/******************************************/

};








/*********************************************************************/
/*                                                                   */
/* Career types                                                      */
/*                                                                   */
/*********************************************************************/
typedef int			CAREER_INDEX;
#define CAREER_NIL		-1

typedef int			CAREER_EXIT_INDEX;

typedef int			N_CAREERS;
typedef int			CAREER_COUNTER;

typedef int			CAREER_TYPE_COUNTER;
typedef int			CAREER_EXIT_COUNTER;

typedef int			N_PERKS;
typedef int			PERK_COUNTER;

/*
 * career
 */
struct career_struct {

	char		name[CAREER_NAME_SIZE];

	CAREER_TYPE	type;

	COLOR		color;

	STAT_MODIFIER	max_stat_advance[MAX_STATS];

	bool 		perk[MAX_PERKS];

	LIST *		trappings;

	char		exit[MAX_CAREER_EXITS][CAREER_NAME_SIZE];

};







/*********************************************************************/
/*                                                                   */
/* Character types                                                   */
/*                                                                   */
/*********************************************************************/
typedef int				NPC_INDEX;
#define NPC_RANDOM_HUMAN		-2

typedef int				N_NPCS;

typedef int				N_CHARACTERS;

typedef int				EXPERIENCE_POINTS;

typedef int				POWER_RATING;

typedef int				INJURY;

typedef int				BLOODY_FEET_VALUE;

typedef int				CONCENTRATION_VALUE;

typedef int				CHARACTER_FLAG_COUNTER;

typedef int				PSY_POWER_COUNTER;



/*
 * character armour rating
 */
struct armour_rating_struct {

	ARMOUR_VALUE	current;
	ARMOUR_VALUE	total;

};

/*
 * character AI data
 */
struct ai_data_struct {

	/*
	 * the currrent state of the AI
	 */
	AI_STATE		state;

	/*
	 * the default state of the AI i.e. the basic state to which
	 * the AI jumps back, once it has finished its current task
	 */
	AI_STATE		default_state;

	/*
	 * the AI state which was set at character creation
	 */
	AI_STATE		original_state;

	/*
	 * the current AI tactic
	 */
	AI_TACTIC		tactic;

	/*
	 * the current target of the AI
	 */
	AREA_POINT		target_point;

};

/*
 * character structure
 */
struct character_struct {

	/* representation data ****************************/
	char			name[CHARACTER_NAME_SIZE];

	SYMBOL			symbol;
	COLOR			color;

	GENDER			gender;
	/**************************************************/


	/* state data *************************************/
	CHARACTER_TYPE		type;
	CHARACTER_CONTROLLER 	controller;

	AREA_POINT		location;

	bool			is_hostile;

	INJURY			injury;

	BLOODY_FEET_VALUE	bloody_feet;

	CONCENTRATION_VALUE	concentration;

	bool			action_spent;
	/**************************************************/


	/* stats ******************************************/
	CHARACTER_STAT		stat[MAX_STATS];
	/**************************************************/


	/* flags, perks, psychic powers *******************/
	bool			flag[MAX_CHARACTER_FLAGS];

	bool			perk[MAX_PERKS];

	bool			psy_power[MAX_PSY_POWERS];
	/**************************************************/


	/* advancement data *******************************/
	CAREER_INDEX		career;

	EXPERIENCE_POINTS	ep;
	EXPERIENCE_POINTS	ep_total;
	/**************************************************/


	/* party data *************************************/
	PARTY			party;

	bool			recruitable;
	/**************************************************/


	/* inventory, equipment ****************************/
	LIST *			inventory;

	OBJECT *		weapon;
	OBJECT *		secondary_weapon;
	OBJECT *		armour;
	OBJECT *		jump_pack;

	ARMOUR_RATING		armour_rating;
	/**************************************************/


	/* perception data ********************************/
	LIST *			noticed_enemies;
	/**************************************************/


	/* AI data ****************************************/
	AI_DATA			ai;
	/**************************************************/


	/* serialization data *****************************/
	LIST_NODE *		list_node;
	LIST_NODE_INDEX		list_index;
	/**************************************************/


	/* source file ************************************/
	FILE_INDEX		source_file;
	/**************************************************/

};


