/*
 * Copyright (C) 2002-2007 The Warp Rogue Team
 * Part of the Warp Rogue Project
 *
 * This software is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License.
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY.
 *
 * See the license.txt file for more details.
 */

/*
 * Module Name: Ui
 * Description: -
 */

/*
 * Misc. stuff
 */

#define Uses_Util
#define Uses_Game
#define Uses_Stats
#define Uses_HallOfFame
#define Uses_Macro
#define Uses_World
#define Uses_DataFile
#define Uses_Inventory
#include "mheader.h"

#include "ui.h"

#include "rng.h"

static void		render_dialog_screen_active_window(DIALOG_STATE *);
static void		render_dialog_screen_passive_window(DIALOG_STATE *);



/*
 * renders the title screen
 */
void render_title_screen(const char *warning_message)
{
	SCREEN_COORD row, col;

	render_secondary_screen("-== %s ==-", GAME_NAME);

	render_logo();

	row = SEC_SCREEN_MIN_Y + 10;
	col = SEC_SCREEN_MIN_X + 2;

	render_text_at(C_TEXT, row, col,
		"A gothic science fantasy roguelike game"
	);
	render_text_at(C_TEXT, ++row, col,
		"Official website: http://todoom.sourceforge.net"
	);

	render_text_at(C_TEXT, row += 3, col,
		"Version %s", GAME_VERSION
	);


	row = SEC_SCREEN_MAX_Y - 1;

 	render_text_at(C_TEXT, row, col, "Copyright (C) 2002-2007 The Warp Rogue Team");

	if (warning_message == NULL) {

		return;
	}

	render_text_at(C_WARNING,
		SEC_SCREEN_MAX_Y - 3, SEC_SCREEN_MIN_X + 2,
		warning_message
	);
}



/*
 * title screen menu
 */
COMMAND title_screen_menu(MENU *menu)
{
	SCREEN_AREA menu_area = {
		SEC_SCREEN_MIN_Y + 17, MENU_AUTO,
		SEC_SCREEN_MIN_X + 2, MENU_AUTO
	};

	menu_set_appearance(menu, &menu_area);

	return menu_execute(menu, &menu_area);
}



/*
 * new game dialog
 */
void new_game_dialog(CONFIRMATION_DIALOG *dialog)
{

	standard_confirmation_dialog(dialog,
		SEC_SCREEN_MAX_Y - 3,
		SEC_SCREEN_MIN_X + 2
	);
}



/*
 * renders the choose difficulty screen
 */
void render_choose_difficulty_screen(void)
{
	char difficulty_info[TEXT_BUFFER_SIZE];

	render_secondary_screen("-== Choose Difficulty ==-");

	read_text_file(difficulty_info, FILE_DIFFICULTY_INFO);

	render_long_text(C_TEXT, SEC_SCREEN_MIN_Y, SEC_SCREEN_MIN_X,
		difficulty_info, SEC_SCREEN_MAX_X
	);
}



/*
 * choose difficulty screen menu
 */
COMMAND choose_difficulty_screen_menu(MENU *menu)
{
	SCREEN_AREA menu_area = {
		SEC_SCREEN_MIN_Y + 7, MENU_AUTO,
		SEC_SCREEN_MIN_X, MENU_AUTO
	};

	menu_set_appearance(menu, &menu_area);

	return menu_execute(menu, &menu_area);
}



/*
 * renders the world generation screen
 */
void render_world_generation_screen(bool generation_finished)
{
	const char *generation_message =
		"The program will now generate the game world. " \
		"This might take a while. ";

	render_secondary_screen("-== World Generation ==-");

	render_long_text(C_TEXT, SEC_SCREEN_MIN_Y, SEC_SCREEN_MIN_X,
		generation_message, SEC_SCREEN_MAX_X
	);

	render_text_at(C_TEXT, SEC_SCREEN_MIN_Y + 4, SEC_SCREEN_MIN_X,
		"Generating %d areas, please wait... ", WORLD_N_AREAS
	);
	
	if (generation_finished) {

		render_text(C_TEXT, "Finished!");
	}
}



/*
 * renders the hall of fame screen
 */
void render_hall_of_fame_screen(void)
{
	const HALL_OF_FAME_ENTRY *hof_entry;
	SCREEN_COORD row, col;
	int i;

	render_secondary_screen("-== Hall of Fame ==-");

	row = SEC_SCREEN_MIN_Y;
	col = SEC_SCREEN_MIN_X;
	i = 0;

	while ((hof_entry = hall_of_fame_entry(i)) != NULL) {

		render_text_at(C_TEXT, row++, col, "%d. %s",
			i + 1,
			hof_entry->name
		);

		if (hof_entry->winner) {
			render_text_at(C_TEXT, row++, col + 3, "WINNER");
		} else {
			render_text_at(C_TEXT, row++, col + 3, "KIA");
		}

		render_text_at(C_TEXT, row++, col + 3, "Score:%d", hof_entry->score);

		++i;
	}

}



/*
 * renders the manufacture screen
 */
void render_manufacture_screen(const CHARACTER *character,
	MANUFACTURE_MODE manufacture_mode)
{
	MONEY personal_funds;

	if (manufacture_mode == MM_TRADER ||
		manufacture_mode == MM_ROGUE_TRADER) {

		render_secondary_screen("-== Buy ==-");

	} else {

		render_secondary_screen("-== Manufacture ==-");
	}

	personal_funds = inventory_money(character);

	render_text_at(C_TEXT, SEC_SCREEN_MAX_Y - 1, SEC_SCREEN_MIN_X,
		"Personal funds: %dc", personal_funds
	);
}



/*
 * manufacture screen menu
 */
COMMAND manufacture_screen_menu(MENU *menu)
{
	SCREEN_AREA menu_area = {
		SEC_SCREEN_MIN_Y, SEC_SCREEN_MAX_Y - 3,
		SEC_SCREEN_MIN_X, MENU_AUTO
	};

	menu_set_appearance(menu, &menu_area);

	return menu_execute(menu, &menu_area);
}



/*
 * renders the dialog screen
 */
void render_dialog_screen(DIALOG_STATE *dialog)
{
	render_background();

	render_dialog_screen_passive_window(dialog);

	render_dialog_screen_active_window(dialog);

	render_command_bar(SID_SECONDARY);
}



/*
 * renders the passive window of the dialog screen
 */
static void render_dialog_screen_passive_window(DIALOG_STATE *dialog)
{

	clear_text_window(SEC_SCREEN_TITLE_ROW, SEC_SCREEN_MIN_X,
		SEC_SCREEN_TITLE_ROW, SEC_SCREEN_MAX_X);

	render_text_at(C_TEXT, SEC_SCREEN_TITLE_ROW, SEC_SCREEN_MIN_X,
		dialog->passive_character->name);

	clear_text_window(
		SEC_SCREEN_MIN_Y,
		SEC_SCREEN_MIN_X,
		SEC_SCREEN_MAX_Y / 2,
		SEC_SCREEN_MAX_X
	);

	place_cursor_at(SEC_SCREEN_MIN_Y, SEC_SCREEN_MIN_X);

	render_long_text(C_TEXT, SEC_SCREEN_MIN_Y, SEC_SCREEN_MIN_X,
		dialog->text, SEC_SCREEN_MAX_X
	);

}



/*
 * renders the active window of the dialog screen
 */
static void render_dialog_screen_active_window(DIALOG_STATE *dialog)
{
	NOT_USED(dialog);

	clear_text_window(
		SEC_SCREEN_MAX_Y / 2 + 2,
		SEC_SCREEN_MIN_X,
		SEC_SCREEN_MAX_Y,
		SEC_SCREEN_MAX_X
	);

}



/*
 * dialog screen menu
 */
COMMAND dialog_screen_menu(MENU *menu)
{
	SCREEN_AREA menu_area = {SEC_SCREEN_MAX_Y / 2 + 2, MENU_AUTO, SEC_SCREEN_MIN_X, MENU_AUTO};

	menu_set_appearance(menu, &menu_area);

	return menu_execute(menu, &menu_area);
}



/*
 * renders the character dump screen
 */
void render_character_dump_screen(void)
{

	render_secondary_screen("-== Character Dump ==-");

	render_text_at(C_TEXT, SEC_SCREEN_MIN_Y, SEC_SCREEN_MIN_X,
		"Character dump written to %s.", FILE_CHARACTER_DUMP
	);
}

